@@ -1,8 +1,8 @@
-#!/bin/sh
+#!/bin/bash
 #
-# Copyright(C) 2010  Tetsuro IKEDA
-# Copyright(C) 2010-2019  Kouhei Sutou <kou@clear-code.com>
-# Copyright(C) 2011  Kazuhiko
+# Copyright (C) 2010  Tetsuro IKEDA
+# Copyright (C) 2010-2022  Sutou Kouhei <kou@clear-code.com>
+# Copyright (C) 2011  Kazuhiko
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -37,11 +37,15 @@ case `uname` in
 esac
 
 if [ "$NO_MAKE" != "yes" ]; then
-  MAKE_ARGS=
-  if [ -n "$n_processors" ]; then
-    MAKE_ARGS="-j${n_processors}"
+  if [ -f "${top_build_dir}/build.ninja" ]; then
+    ninja -C "${top_build_dir}" || exit $?
+  else
+    MAKE_ARGS=
+    if [ -n "$n_processors" ]; then
+      MAKE_ARGS="-j${n_processors}"
+    fi
+    make $MAKE_ARGS -C "${top_build_dir}" > /dev/null || exit $?
   fi
-  make $MAKE_ARGS -C ${top_build_dir} > /dev/null || exit 1
 fi
 
 . "${top_build_dir}/config.sh"
@@ -86,22 +90,22 @@ if [ "${mariadb}" = "yes" ]; then
     fi
   fi
   plugins_dir=
-elif [ "${percona}" = "yes" ]; then
-  case ${MYSQL_VERSION} in
-    8.*)
-      plugins_dir="${MYSQL_BUILD_DIR}/lib/plugin"
-      ;;
-    *)
-      plugins_dir="${MYSQL_SOURCE_DIR}/lib/mysql/plugin"
-      ;;
-  esac
 else
   case ${MYSQL_VERSION} in
     8.*)
-      plugins_dir="${MYSQL_BUILD_DIR}/lib/plugin"
+      if [ -d "${MYSQL_BUILD_DIR}/plugin_output_directory" ]; then
+        plugins_dir="${MYSQL_BUILD_DIR}/plugin_output_directory"
+      else
+        plugins_dir="${MYSQL_BUILD_DIR}/lib/plugin"
+      fi
       ;;
     *)
-      plugins_dir="${MYSQL_SOURCE_DIR}/lib/plugin"
+      if [ "${percona}" = "yes" ]; then
+        plugins_dir="${MYSQL_SOURCE_DIR}/lib/mysql/plugin"
+      else
+        plugins_dir="${MYSQL_SOURCE_DIR}/lib/plugin"
+      fi
+
       ;;
   esac
 fi
@@ -155,11 +159,22 @@ all_test_suite_names=""
 suite_dir="${mroonga_test_dir}/.."
 cd "${suite_dir}"
 suite_dir="$(pwd)"
-for test_suite_name in \
-  $(find mroonga -type d -name 'include' '!' -prune -o \
-         -type d '!' -name 'mroonga' \
-         '!' -name 'include' \
-         '!' -name '[tr]'); do
+find_conditions=(-type d)
+find_conditions+=('(')
+find_conditions+=(-name 'include')
+case ${MYSQL_VERSION} in
+  8.*)
+    find_conditions+=('-o' -name 'optimization')
+    find_conditions+=('-o' -name 'wrapper')
+    ;;
+esac
+find_conditions+=(')')
+find_conditions+=(-prune -o)
+find_conditions+=(-type d)
+find_conditions+=('!' -name 'mroonga')
+find_conditions+=('!' -name '[tr]')
+find_conditions+=(-print)
+for test_suite_name in $(find mroonga "${find_conditions[@]}"); do
   if [ -n "${all_test_suite_names}" ]; then
     all_test_suite_names="${all_test_suite_names},"
   fi
@@ -186,7 +201,7 @@ while [ $# -gt 0 ]; do
   arg="$1"
   shift
   case "$arg" in
-    --manual-gdb|--gdb|--client-gdb|--boot-gdb|--debug|--valgrind)
+    --manual-gdb|--gdb|--client-gdb|--boot-gdb|--debug|--valgrind|--start-and-exit)
       n_processors=1
       mysql_test_run_options="${mysql_test_run_options} ${arg}"
       ;;
